// (C) 2013-2014 Roger Villemaire <villemaire.roger@uqam.ca>

package transport;

public class Camion {
    protected static int noLivraisonsMax = 10; //Taille max d'un camion.

    protected Chauffeur chauffeur;
    protected int poids; // Le poids total du chargement (toutes les livraisons).
    protected int poidsMax; // Le poids max du chargement.
    protected Livraison[] livraisons; // Un tableau formé des articles avec leurs destinations.
    protected int prochaineLivraison; // L'index du prochain espace vide du tableau précédent.


    public Camion(int poidsMax){
	poids = 0;
        chauffeur = new Chauffeur();
        this.poidsMax = poidsMax;
        livraisons = new Livraison[noLivraisonsMax]; // 'noLivraisonsMax' Livraisons au maximum.
        prochaineLivraison = 0;
    }

    public void setChauffeur(Chauffeur c){
	chauffeur = c;
    }

    /*@
      public normal_behavior
      requires this != null;
      assignable chauffeur, c.camion;
      ensures chauffeur == c && c.camion == this;
    @*/
    public void estConduitPar(Chauffeur c){
        // S'assure que *this* est conduit par 'c' et que 'c' conduit *this*.
        chauffeur = c;
        chauffeur.setCamion(this);
    }

   /*@
      public exceptional_behavior 
      requires poids + article.poids <= poidsMax; //Le poidsMax n'est pas dépassé.
      requires prochaineLivraison == noLivraisonsMax; //Plus de place dans le camion.
      signals (ExcCamionPlein) prochaineLivraison == \old(prochaineLivraison);
      signals_only ExcCamionPlein;
    @*/
    public boolean charger(Article article, Lieu destination) throws ExcCamionPlein {
        if ( poids + article.getPoids() > poidsMax ){
            return false;
        }
        else {
            if (prochaineLivraison == noLivraisonsMax){ //Plus de place
                throw new ExcCamionPlein();
            }
            else{
                poids += article.getPoids();
            }
            // Insérer la livraison et incrémenter l'index.
	    Livraison l = new Livraison(article,destination);
	    livraisons[prochaineLivraison] = l;
            prochaineLivraison++;

            return true;
        }
    }
}